"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validate = validate;
exports.qs = qs;
exports.forEach = forEach;
exports.kebabCase = kebabCase;
exports.camelCase = camelCase;
exports.displayErrors = displayErrors;
exports.displayWarnings = displayWarnings;

/**
 * Check validity
 *
 * @export
 * @param {object} variables data
 * @throws {Error} if error found
 * @returns {undefined}
 */
function validate(variables) {
  // For components, trigger means "auto" submit
  // so values can not be passed…
  forEach(variables.components, function (component) {
    Object.keys(component) // Filter if optional trigger
    .filter(function (layer) {
      return component[layer].trigger;
    }).forEach(function (layer) {
      // Prepare data
      var data = component[layer];
      forEach(data, function (value) {
        if (value.match(/^\$val/)) {
          throw new Error('🚨 Values ($val…) are not allowed for components with "trigger"');
        }
      });
    });
  });
}
/**
 * Get element by CSS selector.
 *
 * @param {String} selector CSS selector
 * @param {HTMLElement} scope DOM element to query inside
 * @returns {HTMLElement} queried DOM element
 */


function qs(selector, scope) {
  return (scope || document).querySelector(selector);
}
/**
 * ForEach for array/object (basic version)
 *
 * @param {Array|Object} collection "collection" to loop over
 * @param {Function} iteratee callback
 * @returns {Array|Object} initial collection
 */


function forEach(collection, iteratee) {
  if (collection.forEach === undefined) {
    Object.keys(collection).forEach(function (key) {
      iteratee(collection[key], key);
    });
  } else {
    collection.forEach(iteratee);
  }

  return collection;
}
/**
* Convert camelCase to kebab-case
*
* @param {String} value original
* @returns {String} kebab-cased
*/


function kebabCase(value) {
  return value.toString().replace(/([A-Z])/g, function (_, char) {
    return "-".concat(char.toLowerCase());
  });
}
/**
* Convert kebab-case to camelCase
*
* @param {String} value original
* @returns {String} camel-cased
*/


function camelCase(value) {
  return value.toString().replace(/(-[a-z])/g, function (_, char) {
    return "".concat(char.replace('-', '').toUpperCase());
  });
}
/**
 * Display errors.
 *
 * @static
 * @param {string | Error | array} fail error(s) or message(s)
 * @returns {undefined}
 */


function displayErrors(fail) {
  var fails = Array.isArray(fail) ? fail : [fail];
  var errors = [];
  fails.forEach(function (fail) {
    if (typeof fail === 'string') {
      errors.push(new Error("\uD83D\uDCC8 ".concat(fail, "\nFor more informations, see https://github.com/epicagency/snitchy")));
    } else {
      errors.push(fail);
    }
  }); // DEV find replacement
  // throw new AggregateError(errors);

  throw new Error(errors[0]);
}
/**
 * Display warnings.
 *
 * @static
 * @param {string} message custom message
 * @returns {undefined}
 */


function displayWarnings(message) {
  console.warn("\uD83D\uDCC8 ".concat(message, "\nFor more informations, see\u2026(documentation link)"));
}